==> netw-ib-ox-ag_5.30.0.orig.tar.gz: netw-ib-ox-ag-5.30.0/src/netwib-src/src/netwib/sys/priv/dir.c <==
/* This file is part of Netwib.
*/

/*-------------------------------------------------------------*/
netwib_err netwib_priv_dir_create(netwib_constbuf *pdirname)
{
  netwib_string dirname;
  netwib_bool errwin240;
  int reti;

  netwib__buf_ref_string(pdirname, dirname, bufstorage,
                         netwib_priv_dir_create(&bufstorage));

  errwin240 = NETWIB_FALSE;
#if defined NETWIBDEF_SYSNAME_Unix
  reti = mkdir(dirname, 0700);
#elif defined NETWIBDEF_SYSNAME_Windows
  reti = _mkdir(dirname);
  if (reti == -1) {
    if (errno == ENOENT) {
      errwin240 = NETWIB_TRUE;
    }
  }
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  if (reti == -1) {
    netwib_er(netwib_priv_errmsg_text("cannot create this dir: "));
    netwib_er(netwib_priv_errmsg_append_buf(pdirname));
#if defined NETWIBDEF_SYSNAME_Unix
#elif defined NETWIBDEF_SYSNAME_Windows
    if (errwin240) {
      netwib_er(netwib_priv_errmsg_append_text(" (origin of this problem is perhaps Windows'limitation of 240 chars for total pathname length)"));
    }
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif
    return(NETWIB_ERR_FUMKDIR);
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_dir_create_recur(netwib_constbuf *pdirname)
{
  netwib_byte array[512];
  netwib_buf parent;
  netwib_bool yes;
  netwib_err ret;

  netwib_er(netwib_buf_init_ext_arrayempty(array, sizeof(array), &parent));
  parent.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  ret = netwib_path_decode_parent(pdirname, &parent);
  if (ret == NETWIB_ERR_PAPATHCANTINIT) {
    /* can't go up, we will directly create directory */
  } else if (ret != NETWIB_ERR_OK) {
    netwib_er(netwib_buf_close(&parent));
    return(ret);
  } else {
    /* eventually create parent */
    netwib_er(netwib_dirname_exists(&parent, &yes));
    if ( !yes ) {
      netwib_priv_dir_create_recur(&parent);
    }
  }
  netwib_er(netwib_buf_close(&parent));

  netwib_er(netwib_priv_dir_create(pdirname));
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_dir_create_parents(netwib_constbuf *ppathname)
{
  netwib_byte parentarray[512], startarray[128];
  netwib_buf parent, start;
  netwib_bool yes;
  netwib_err ret;

  /* obtain parent */
  netwib_er(netwib_buf_init_ext_arrayempty(parentarray, sizeof(parentarray),
                                           &parent));
  parent.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  ret = netwib_path_decode_parent(ppathname, &parent);
  if (ret == NETWIB_ERR_PAPATHCANTINIT) {
    /* can't go up */
    netwib_er(netwib_buf_close(&parent));
    return(NETWIB_ERR_OK);
  } else if (ret != NETWIB_ERR_OK) {
    netwib_er(netwib_buf_close(&parent));
    return(ret);
  }

  /* if parent exists, do not need to continue */
  netwib_er(netwib_dirname_exists(&parent, &yes));
  if ( yes ) {
    netwib_er(netwib_buf_close(&parent));
    return(NETWIB_ERR_OK);
  }

  /* check if begin directory exists */
  netwib_er(netwib_buf_init_ext_arrayempty(startarray, sizeof(startarray),
                                           &start));
  start.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  netwib_er(netwib_path_decode_begin(&parent, &start));
  netwib_er(netwib_dirname_exists(&start, &yes));
  if ( !yes ) {
    netwib_er(netwib_priv_errmsg_text("cannot create this dir: "));
    netwib_er(netwib_priv_errmsg_append_buf(&start));
    netwib_er(netwib_buf_close(&start));
    netwib_er(netwib_buf_close(&parent));
    return(NETWIB_ERR_NOTFOUND);
  }
  netwib_er(netwib_buf_close(&start));

  /* create parent */
  netwib_er(netwib_priv_dir_create_recur(&parent));
  netwib_er(netwib_buf_close(&parent));

  return(NETWIB_ERR_OK);
}

==> netw-ib-ox-ag_5.34.0.orig.tar.gz: netw-ib-ox-ag-5.34.0/src/netwib-src/src/netwib/sys/priv/dir.c <==
/* This file is part of Netwib.
*/

/*-------------------------------------------------------------*/
netwib_err netwib_priv_dir_create(netwib_constbuf *pdirname)
{
  netwib_string dirnamestr;
  netwib_bool errwin240;
  int reti;

  netwib__buf_ref_string(pdirname, dirnamestr, bufstorage,
                         netwib_priv_dir_create(&bufstorage));

  errwin240 = NETWIB_FALSE;
#if defined NETWIBDEF_SYSNAME_Unix
  reti = mkdir(dirnamestr, 0700);
#elif defined NETWIBDEF_SYSNAME_Windows
  reti = _mkdir(dirnamestr);
  if (reti == -1) {
    if (errno == ENOENT) {
      errwin240 = NETWIB_TRUE;
    }
  }
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif

  if (reti == -1) {
    netwib_er(netwib_priv_errmsg_text("cannot create this dir: "));
    netwib_er(netwib_priv_errmsg_append_buf(pdirname));
#if defined NETWIBDEF_SYSNAME_Unix
#elif defined NETWIBDEF_SYSNAME_Windows
    if (errwin240) {
      netwib_er(netwib_priv_errmsg_append_text(" (origin of this problem is perhaps Windows'limitation of 240 chars for total pathname length)"));
    }
#else
#error "Unknown value for NETWIBDEF_SYSNAME"
#endif
    return(NETWIB_ERR_FUMKDIR);
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
static netwib_err netwib_priv_dir_create_recur(netwib_constbuf *pdirname)
{
  netwib_byte array[512];
  netwib_buf parent;
  netwib_bool yes;
  netwib_err ret;

  netwib_er(netwib_buf_init_ext_arrayempty(array, sizeof(array), &parent));
  parent.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  ret = netwib_path_decode_parent(pdirname, &parent);
  if (ret == NETWIB_ERR_PAPATHCANTINIT) {
    /* can't go up, we will directly create directory */
  } else if (ret != NETWIB_ERR_OK) {
    netwib_er(netwib_buf_close(&parent));
    return(ret);
  } else {
    /* eventually create parent */
    netwib_er(netwib_dirname_exists(&parent, &yes));
    if ( !yes ) {
      netwib_priv_dir_create_recur(&parent);
    }
  }
  netwib_er(netwib_buf_close(&parent));

  netwib_er(netwib_priv_dir_create(pdirname));
  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_dir_create_parents(netwib_constbuf *ppathname)
{
  netwib_byte parentarray[512], startarray[128];
  netwib_buf parent, start;
  netwib_bool yes;
  netwib_err ret;

  /* obtain parent */
  netwib_er(netwib_buf_init_ext_arrayempty(parentarray, sizeof(parentarray),
                                           &parent));
  parent.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  ret = netwib_path_decode_parent(ppathname, &parent);
  if (ret == NETWIB_ERR_PAPATHCANTINIT) {
    /* can't go up */
    netwib_er(netwib_buf_close(&parent));
    return(NETWIB_ERR_OK);
  } else if (ret != NETWIB_ERR_OK) {
    netwib_er(netwib_buf_close(&parent));
    return(ret);
  }

  /* if parent exists, do not need to continue */
  netwib_er(netwib_dirname_exists(&parent, &yes));
  if ( yes ) {
    netwib_er(netwib_buf_close(&parent));
    return(NETWIB_ERR_OK);
  }

  /* check if begin directory exists */
  netwib_er(netwib_buf_init_ext_arrayempty(startarray, sizeof(startarray),
                                           &start));
  start.flags |= NETWIB_BUF_FLAGS_CANALLOC;
  netwib_er(netwib_path_decode_begin(&parent, &start));
  netwib_er(netwib_dirname_exists(&start, &yes));
  if ( !yes ) {
    netwib_er(netwib_priv_errmsg_text("cannot create this dir: "));
    netwib_er(netwib_priv_errmsg_append_buf(&start));
    netwib_er(netwib_buf_close(&start));
    netwib_er(netwib_buf_close(&parent));
    return(NETWIB_ERR_NOTFOUND);
  }
  netwib_er(netwib_buf_close(&start));

  /* create parent */
  netwib_er(netwib_priv_dir_create_recur(&parent));
  netwib_er(netwib_buf_close(&parent));

  return(NETWIB_ERR_OK);
}
